import getopt,sys,os,json,time
import urllib.request
import urllib.parse
from datetime import datetime as dt

# version for Python 3.8

def usage():
	print("\nUsage: " + sys.argv[0])
	print("\t-h display this notice")
	print("\t-D domain")
	print("\t-u username")
	print("\t-p password")
	print("\t-a action [index|get]")
	print("\t-d target directory [required if -a get]")
	print("\t-k keep files on server [used if -a get]")
	print("\t-s sleep timer [optional], default: 0.5")
	print("\t-i skip already downloaded files, default: no (if -k is not set, delete the files from the server regardless if downloaded or not)")
	print("\t-I insecure disable HTTPS")
	print("\t-t save the time of the recording that was downloaded last and ignore older recordings on the server for the next downloads")

class getOptions():
	def Initialize(self):
		opts, args = getopt.getopt(sys.argv[1:], 'hD:u:p:a:d:ks:iIt')

		config = {
			'domain': None,
			'username': None,
			'password': None,
			'action': None,
			'targetDir': None,
			'keep': False,
			'sleep': 0.5,
			'override': True,
			'protocol': 'https://',
			'saveLastTime': False,
			'timeFileName': 'lastDownloadedDateTime.txt',
			'dtFormat': '%Y-%m-%d %H:%M:%S'
		}

		requiredConfig = {
			'-D': 'domain',
			'-u': 'username',
			'-p': 'password',
			'-a': 'action'
		}

		for opt, arg in opts:
			if opt == '-h':
				usage()
				sys.exit()
			if opt == '-D':
				config['domain'] = arg
			elif opt == '-u':
				config['username'] = arg
			elif opt == '-p':
				config['password'] = arg
			elif opt == '-a':
				config['action'] = arg
			elif opt == '-d':
				config['targetDir'] = arg
			elif opt == '-k':
				config['keep'] = True
			elif opt == '-s':
				arg = float(arg)
				if arg >= 0:
					config['sleep'] = arg
				else:
					config['sleep'] = 0
			elif opt == '-i':
				config['override'] = False
			elif opt == '-I':
				config['protocol'] = 'http://'
			elif opt == '-t':
				config['saveLastTime'] = True

		missingRequiredOpts = []
		for opt, index in requiredConfig.items():
			if config[index] == None:
				missingRequiredOpts.append(opt)

		if config['action'] == 'get' and config['targetDir'] == None:
			missingRequiredOpts.append('-d')

		if len(missingRequiredOpts) > 0:
			print("\nMissing required flags: ", end = '')
			print(*missingRequiredOpts, sep = ', ')
			sys.exit()

		return config

class Recordings():
	#####################################
	# Connections to remote .php script #
	#####################################

	# function to get an encoded JSON array and returns decoded object
	def getJSON(self, url):
		req = urllib.request.Request(url)
		opener = urllib.request.build_opener()
		f = opener.open(req)
		return json.load(f)

	# function to get results from a URL that doesn't return a JSON object
	def getURL(self, url):
		req = urllib.request.Request(url)
		response = urllib.request.urlopen(req)
		return response.read()

	########################################
	# Functions for copying recorded calls #
	########################################

	# function to create a folder if it does not exist yet
	def createFolder(self, path):
		print("Checking if " + path + " exists ...")
		if not os.path.isdir(path):
			print("    Creating " + path + " ...\n")
			os.makedirs(path)
		else:
			print("    " + path + " exists, skipping ...\n")
		return path

	# function to create a file at path/file-name
	def createFile(self, baseUrl, fileId, localPath, fileName, override):
		fileName = fileName.replace('*', '')
		filePath = os.path.join(localPath, fileName)

		print("Checking if " + filePath + " exists ...\n")
		if not os.path.exists(filePath) or override:
			newFilePath = self.getNewFilePath(filePath)

			print("    Creating " + newFilePath + " ...\n")
			contents = self.getURL(baseUrl + "&action=download&file_id=" + urllib.parse.quote_plus(fileId))
			with open(newFilePath, 'wb') as f:
				f.write(contents)
				f.close()
		else:
			print("    Skipping " + filePath + " ...\n")

	# function to check if file already exists and returns an alternative file name since we do not want to override existing files
	def getNewFilePath(self, path, append = 0):
		if os.path.exists(path):
			append += 1
			splitPath = path.rsplit('.')
			nextPath = splitPath[0] + "-" + str(append) + "." + splitPath[1]
			print("    " + path + " exists, checking " + nextPath + " ...")
			checkPath = self.getNewFilePath(nextPath, append)

		return path

	# function to get data from a file ID
	def getFileData(self, path):
		path = path.replace('~','/')
		path = path.replace('#AUTO#','~')
		splitPath = path.rsplit('/')
		fileData = splitPath[-1].rsplit('__')
		fileData[0] = fileData[0].rsplit('~')[-1]

		folder = ''
		i = 0
		while i < len(splitPath) - 1:
			folder += splitPath[i] + "/"
			i += 1
		fileName = splitPath[i] + "__" + config['username'] + ".wav"

		return {
			'path': path,
			'splitPath': splitPath,
			'folder': folder,
			'fileName': fileName,
			'date': splitPath[-2] + " " + fileData[0].replace('-', ':'),
			'caller': fileData[1],
			'called': fileData[2]
		}

	##########################################
	# Functions for deleting files on server #
	##########################################

	# function to delete paths on server; if path is a folder, it's only deleted if it's empty
	def deleteFile(self, baseUrl, fileId, localPath):
		# check if the file or folder has been copied before we delete it
		if os.path.exists(localPath):
			contents = self.getURL(baseUrl + "&action=delete&file_id=" + urllib.parse.quote_plus(fileId))
			if(contents != ''):
				print(contents)
			else:
				print(fileId + " deleted successfully on remote server!\n")
		else:
			print(localPath + " does not yet exist, skipping the deleting process of file " + fileId + ". Please restart this script to try again.\n")

	#################################
	# Functions according to action #
	#################################

	# function to check recordings saved at src
	def showContents(self, baseUrl, config):
		contents = self.getJSON(baseUrl + "&action=index")
		print("\nThese files are available:")
		if len(contents) > 0:
			checkTime = self.getCheckTime(config)

			for fileId in contents:
				fileData = self.getFileData(fileId)
				if config['saveLastTime'] and checkTime != None:
					thisTime = dt.strptime(fileData['date'], config['dtFormat'])
					if thisTime <= checkTime:
						continue
				print("    " + fileData['date'] + " from " + fileData['caller'] + " to " + fileData['called'])
		else:
			print("    <none>")

	# function to move folders and files from server to destination
	def moveContents(self, baseUrl, config):
		print("\nStarting to copy files ...")
		contents = self.getJSON(baseUrl + "&action=get")
		if len(contents) > 0:
			checkTime = self.getCheckTime(config)
			saveTime = None

			for fileId in contents:
				fileData = self.getFileData(fileId)

				if config['saveLastTime']:
					thisTime = dt.strptime(fileData['date'], config['dtFormat'])
					if checkTime != None and thisTime <= checkTime:
						continue

					if saveTime == None or thisTime > saveTime:
							saveTime = thisTime

				# getting valid local path and copy
				localPath = os.path.join(config['targetDir'], os.path.normpath(fileData['folder']))
				newFolderPath = self.createFolder(localPath)
				self.createFile(baseUrl, fileId, newFolderPath, fileData['fileName'], config['override'])

				# delete file
				if not config['keep']:
					localPath = os.path.join(localPath, fileData['fileName'])
					self.deleteFile(baseUrl, fileId, localPath)

				# some systems cause problems when there are too many downloads one after another
				time.sleep(config['sleep'])

			if config['saveLastTime'] and saveTime != None:
				with open(config['timeFileName'], 'w') as f:
					f.write(saveTime.strftime(config['dtFormat']))
					f.close()
		else:
			print("    No files available!")

	def getCheckTime(self, config):
		checkTime = None
		if config['saveLastTime'] and os.path.exists(config['timeFileName']):
			with open(config['timeFileName'], 'r') as f:
				lastTime = f.read()
				if lastTime != '':
					checkTime = dt.strptime(lastTime, config['dtFormat'])
				f.close()

		return checkTime

config = getOptions().Initialize()
baseUrl = config['protocol'] + config['domain'] + "/recordings.php?username=" + config['username'] + "&password=" + config['password']

if config['action'] == 'index':
	Recordings().showContents(baseUrl, config)
elif config['action'] == 'get':
	Recordings().moveContents(baseUrl, config)
else:
	print("\nUnkown action.")